﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Pendulum : MonoBehaviour {

	[Header("Values")]
	public bool activate = false;
	public float swingSpeed = 60;

	[Header("Axis X")]
	public bool swingAxisX = false;
	public int directionAxisX = 1;
	public int minAxisX, maxAxisX;

	[Header("Axis Z")]
	public bool swingAxisZ = false;
	public int directionAxisZ = 1;
	public int minAxisZ, maxAxisZ;

	[Header("Attack")]
	public AttackArea attackObject;

	Vector3 defaultSwingAxis;
	int swingCounter;

	Vector3 currentRotate;

	[Header("Time when ignore any activate request")]
	public float ignoreTime = 1.5f;
	float ignoreTimeCounter = 0;


	// Use this for initialization
	void Start () {
		currentRotate = transform.rotation.eulerAngles;
		if ( currentRotate.x > 180)
		{
			currentRotate.x -= 360;
		}
		if ( currentRotate.z > 180)
		{
			currentRotate.z -= 360;
		}

		defaultSwingAxis = currentRotate;
	}
	
	// Update is called once per frame
	void FixedUpdate () {
		
		if ( activate == true && holdStill == true)
		{
			activate = false;
			Activate();	
		}


		Swinging();

		if (ignoreTimeCounter > 0)
		{
			ignoreTimeCounter -= Time.fixedDeltaTime;
		}
	}

	void Activate()
	{
		if ( ignoreTimeCounter > 0)
		{
			return;
		}

		holdStill = false;

		swingCounter = 2;
		ignoreTimeCounter = ignoreTime;

		if ( attackObject != null)
		{
			attackObject.Activate();
		}
	}


	int GetDirection(float currentRotation, int currentDirection, int minSwing, int maxSwing )
	{
		if ( currentDirection == 1)
		{
			if ( currentRotation < maxSwing )
			{
				return 1;
			}
			else
			{
				return -1;
			}
		}
		else
		{
			if ( currentRotation < minSwing )
			{
				return 1;
			}
			else
			{
				return -1;
			}
		}

	}
		
	void Swinging()
	{
		if ( swingCounter > 0)
		{
			DoFullSwing();
		}
		else if (holdStill == false)
		{
			DoReset();
		}
	}

	[HideInInspector]
	public bool holdStill = true;
	float swingToResetCounter = 0;
	void DoReset()
	{
		if ( swingAxisX == true)
		{
			swingToResetCounter += Time.fixedDeltaTime;
			float fullDistance = Mathf.Abs( currentRotate.x - defaultSwingAxis.x)/swingSpeed;

			float proportion = swingToResetCounter/fullDistance;

			float currentRotationValue = Mathf.LerpAngle(currentRotate.x, defaultSwingAxis.x, proportion);

			transform.rotation = Quaternion.Euler( currentRotationValue, currentRotate.y, currentRotate.z);
			if (proportion >= 1)
			{
				holdStill = true;
			}
		}
		else if ( swingAxisZ == true)
		{
			swingToResetCounter += Time.fixedDeltaTime;
			float fullDistance = Mathf.Abs( currentRotate.z - defaultSwingAxis.z)/swingSpeed;

			float proportion = swingToResetCounter/fullDistance;

			float currentRotationValue = Mathf.LerpAngle(currentRotate.z, defaultSwingAxis.z, proportion);

			transform.rotation = Quaternion.Euler( currentRotate.x, currentRotate.y, currentRotationValue);
			if (proportion >= 1)
			{
				holdStill = true;
			}
		}
	}

	void DoFullSwing()
	{
		if ( swingAxisX == true)
		{
			int newDirection = GetDirection(currentRotate.x, directionAxisX, minAxisX, maxAxisX);
			if ( newDirection != directionAxisX)
			{
				 ChangeDirection();
			}

			float rotateDegree = Time.fixedDeltaTime * swingSpeed * directionAxisX;

			transform.Rotate(rotateDegree,0,0);
			currentRotate = transform.rotation.eulerAngles;
		}
		else if ( swingAxisZ == true)
		{
			int newDirection = GetDirection(currentRotate.z, directionAxisZ, minAxisZ, maxAxisZ);
			if ( newDirection != directionAxisZ)
			{
				ChangeDirection();
			}

			float rotateDegree = Time.fixedDeltaTime * swingSpeed * directionAxisZ;

			transform.Rotate(0,0,rotateDegree);
			currentRotate = transform.rotation.eulerAngles;
		}

		if ( currentRotate.x > 180)
		{
			currentRotate.x -= 360;
		}
		if ( currentRotate.z > 180)
		{
			currentRotate.z -= 360;
		}
	}

	void ChangeDirection()
	{
		if (swingAxisX == true)
		{
			directionAxisX = -directionAxisX;
		}
		else if ( swingAxisZ == true)
		{
			directionAxisZ = -directionAxisZ;
		}
		else 
		{
			return;
		}

		swingCounter--;
		swingToResetCounter = 0;

	}

	public int hitPeriod = 0;

	public void BallHitPendulum(Transform other)
	{
		
		Pendulum node = other.GetComponent<Pendulum>();
		if (node != null )
		{
			if ( node.holdStill == true)
			{
				UpdateResultWhenAnotherPendulumNotMoving(node);	
			}
			else if ( node.holdStill == false && holdStill == false)
			{
				UpdateResultWhenAnotherPendulumMoving(node);
			}

		}
	}

	void UpdateResultWhenAnotherPendulumNotMoving(Pendulum other)
	{
		other.UpdateAxisFromCollision(transform);
		other.activate = true;

		swingCounter = 1;
		ChangeDirection();

	}

	void UpdateResultWhenAnotherPendulumMoving(Pendulum other)
	{
		if ( swingCounter <= 0)
		{
			Activate();
		}

		swingCounter = 1;
		ChangeDirection();
	}

	void SwitchToAxisX()
	{
		swingAxisZ = false;
		swingAxisX = true;
	}

	void SwitchToAxisZ()
	{
		swingAxisZ = true;
		swingAxisX = false;
	}

	public void UpdateAxisFromCollision(Transform other)
	{
		Pendulum node = other.GetComponent<Pendulum>();
		if (node != null)
		{
			if ( node.swingAxisX == true)
			{
				directionAxisX = node.directionAxisX;
				SwitchToAxisX();
			}
			else if ( node.swingAxisZ == true)
			{
				directionAxisZ = node.directionAxisZ;
				SwitchToAxisZ();
			}
				

		}
	}


}
